/* ------------------------------------------------------------------------- */
/*   Copyright (C) 2009 Eric J. Gaudet

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.		     */
/* ------------------------------------------------------------------------- */

function FirstAssistant() {
	/* this is the creator function for your scene assistant object. It will be passed all the 
	   additional parameters (after the scene name) that were passed to pushScene. The reference
	   to the scene controller (this.controller) has not be established yet, so any initialization
	   that needs the scene controller should be done in the setup function below. */
}

FirstAssistant.prototype.setup = function() {
	/* this function is for setup tasks that have to happen when the scene is first created */
		
	/* use Mojo.View.render to render view templates and add them to the scene, if needed. */
	
        $('mygroup').hide();

        /* boundaries array (xleft, xright, ytop, ybottom) */
        
        this.full_offset = 0;
        this.full_offset = 50;
        var bot = 395 + this.full_offset;
        this.stopObjects = [
          /* Screen */
          {x: {start: -1000, end: 0},    y: {start: -1000, end: 1000}},
          {x: {start:  290,  end: 1000}, y: {start: -1000, end: 1000}},
          {x: {start: -1000, end: 1000}, y: {start: -1000, end: 0}},
          {x: {start: -1000, end: 1000}, y: {start:  bot,  end: 1000}},

          /* Railings */
          {x: {start:  10,   end: 1000},  y: {start: 46,   end: 90}},
          {x: {start: -1000, end: 219},   y: {start: 137,  end: 182}},
          {x: {start:  77,   end: 227},   y: {start: 213,  end: 258}},
          {x: {start: -1000, end: 216},   y: {start: 307,  end: 350}},

          {x: {start:  227,  end: 272},   y: {start: 83,   end: 370}}
        ];

        this.actionObjects = [
          {x: 28,  y:39,  win: false},
          {x: 147, y:95,  win: false},
          {x: 99,  y:211, win: false},
          {x: 5,   y:257, win: false},
          {x: 147, y:272, win: false},
          {x: 59,  y:375, win: false},
          {x: 132, y:412, win: false},
          {x: 249, y:392, win: false},
          {x: 289, y:449, win: false},

          {x: 283, y:89, win: true},
          ];

	this.ball = {
            x:  289,
            y:  1,
            ax: 0,
            ay: 0,
            az: 0,
            collision: {}
        };
        
        this.poll_interval = 250;
        this.paused = true;
        this.debug = false;
        
        this.appMenuModel = {
            visible: true,
            items: [
                { label: "Debug", command: 'do-debug'},
                { label: "Restart", command:'do-restart' },
                { label: "Pause", command:'do-pause'},
                { label: "Sensitivity", disabled:true,
                    items: [
                        {label: "low (default)", disabled: true, command: 'sens-low'},
                        {label: "medium", disabled: true, command: 'sens-med'},
                        {label: "high", disabled: true, command: 'sens-high'}
                        ]
                    }
            ]
        };
        
	      this.setFreq(1000);
        if (this.full_offset != 0) {
          this.controller.enableFullScreenMode(true);
        }

        this.controller.setupWidget(Mojo.Menu.appMenu,
            {omitDefaultItems: true}, this.appMenuModel);
        this.controller.setupWidget('mygroup', {}, {});

        /*****************************************************************\
        * TODO: use a timer to see if the user stopped playing the game, 
        *       and then allow screen dim/off again.
        *
        *       WARNING: Enabling this code blocks the display controls 
        *                from dimming/sleeping.  Implement method to 
        *                re-enable when user is idle before using this 
        *                code!
        \*****************************************************************/
        //var parameters = {};
        //parameters.requestBlock = true;
        //parameters.client = Mojo.appName;
        //parameters.subscribe = true;
        //this.blockRequest = 
          //new Mojo.Service.Request('palm://com.palm.display/control',
            //{
            //method: 'setProperty',
            //parameters: parameters
            //});

        Mojo.Event.listen(document, 'acceleration', 
            this.updateAcceleration.bindAsEventListener(this));

        this.intervalID = window.setInterval(this.updateBall.bind(this), 20);

       var debugMe = false;
       /*****************************************************************\
       * Uncomment to debug - show accelerations / ball coordinates
       *
       *   Fix background so screen doesn't shift (7px down) with debug on
       \*****************************************************************/
       //debugMe = true;
       if (debugMe) {
         this.debug = true;
         this.dbgIntervalID = window.setInterval(this.updateDbg.bind(this),100);
         $('mygroup').show();
       }
}

FirstAssistant.prototype.setFreq = function(newfreq) {
   this.poll_interval = 1000/newfreq;
   
   this.controller.serviceRequest('palm://org.webosinternals.accelservice',
      {
      method: 'setPollFreq',
      parameters: {freq:newfreq}
      });
}

FirstAssistant.prototype.activate = function(event) {
	/* put in event handlers here that should only be in effect when this scene is active. For
	   example, key handlers that are observing the document */
  this.unpauseGame(false);
}


FirstAssistant.prototype.deactivate = function(event) {
	/* remove any event handlers you added in activate and do any other cleanup that should happen before
	   this scene is popped or another scene is pushed on top */
  this.pauseGame(false);
}

/*
FirstAssistant.prototype.cancelScreenBlock = function() {
  if (this.blockRequest) {
    this.blockRequest.cancel();
    this.blockRequest = null;
  }
  var parameters = {};
  parameters.requestBlock = false;
  parameters.client = Mojo.appName;

  return new Mojo.Service.Request('palm://com.palm.display/control', 
      {
        method: 'setProperty',
        parameters: parameters
      });
}
*/

FirstAssistant.prototype.cleanup = function(event) {
	/* this function should do any cleanup needed before the scene is destroyed as 
	   a result of being popped off the scene stack */
  
  // Set back to default Palm value of 4 HZ when app closes
  this.setFreq(4);
}

FirstAssistant.prototype.resumeGame = function() {
  if (!this.intervalID) {
    this.intervalID = window.setInterval(this.updateBall.bind(this), 20);
  }
}
    
FirstAssistant.prototype.pauseGame = function(resetBall) {
    if (resetBall) {
        $('myball').hide();
    }
    else {
        this.appMenuModel.items[2].label="Unpause";
    }
    
    window.clearInterval(this.intervalID);
    this.intervalID = null;
    this.paused = true;
    console.log("PAUSED");
}

FirstAssistant.prototype.unpauseGame = function(resetBall) {
    if (resetBall) {
        $('myball').show();
    }
    else {
        this.appMenuModel.items[2].label="Pause";
    }
      
    this.paused = false;
    window.setTimeout(this.resumeGame.bind(this), 500);
    console.log("UNPAUSED");
}

FirstAssistant.prototype.checkStopObjects = function(obj, newx, newy) {
  /* Would I be in this object? */
  if (newx >= obj.x.start && newx <= obj.x.end &&
      newy >= obj.y.start && newy <= obj.y.end) {
    /* Ok, so what direction am I coming from? */
    if (this.ball.x < obj.x.start) {
      this.ball.x = obj.x.start - 1;
      this.ball.collision.x = true;
    }
    if (this.ball.x > obj.x.end) {
      this.ball.x = obj.x.end + 1;
      this.ball.collision.x = true;
    }
    if (this.ball.y < obj.y.start) { 
      this.ball.y = obj.y.start - 1;
      this.ball.collision.y = true;
    }
    if (this.ball.y > obj.y.end) {
      this.ball.y = obj.y.end + 1;
      this.ball.collision.y = true;
    }
  }
}

FirstAssistant.prototype.checkActionObjects = function(obj) {
  if (this.ball.x > (obj.x - 10) && this.ball.x < (obj.x + 10) && 
      this.ball.y > (obj.y - 10) && this.ball.y < (obj.y + 10)) { 
    this.pauseGame(true);
    if (obj.win) {
      var title = "Congratulations!"
    }
    else {
      var title = "Game Over!"
    }
    this.controller.showAlertDialog({
      onChoose: function(value) {
        if (obj.win) {this.nextLevel(value);}
        else {this.restartGame(value);}
      },
      title: title,
      choices: [
        {label: "Restart", value:"restart"},
        {label: "Quit",    value:"quit"}
      ]
    }); 
  }
}
FirstAssistant.prototype.updateBall = function() {
  var newx = this.ball.x + (20 * this.ball.ax);
  var newy = this.ball.y - (20 * this.ball.ay);
  var collision = {x: false, y: false};
  var firstCollision = true;
	
  this.ball.collision = {x: false, y: false};
  for (var i=0; i<this.stopObjects.length; i++) {
    this.checkStopObjects(this.stopObjects[i], newx, newy);
    if (this.ball.collision.x) {
      newx = this.ball.x;
      if (firstCollision) {
        firstCollision = false;
        i=0;
      }
    }
    if (this.ball.collision.y) {
      newy = this.ball.y;
      if (firstCollision) {
        firstCollision = false;
        i=0;
      }
    }
  }

  this.ball.x = newx;
  this.ball.y = newy;

  console.log("UPDATE BALL");
  document.getElementById('myball').style.left=this.ball.x + "px"
	document.getElementById('myball').style.top=this.ball.y + "px"

  /* paused here indicates beginning of game, because this.paused
   * should never be true if we are here otherwise */
  if (this.paused) {
    this.paused = false;
    $('myball').show();
    return;
  }


  for (i=0; i<this.actionObjects.length; i++) {
    this.checkActionObjects(this.actionObjects[i]);
  }
}

FirstAssistant.prototype.updateDbg = function () {
    this.controller.get('mytext').update(
        "X: " + Mojo.Format.formatNumber(parseFloat(this.ball.ax), {"fractionDigits":"2"}) + "   " +
        "Y: " + Mojo.Format.formatNumber(parseFloat(this.ball.ay), {"fractionDigits":"2"}) + "   " +
        "Z: " + Mojo.Format.formatNumber(parseFloat(this.ball.az), {"fractionDigits":"2"}) + "   ");
    
    this.controller.get('mytext2').update(
        "X: " + Mojo.Format.formatNumber(parseFloat(this.ball.x), {"fractionDigits":"2"}) + "   " +
        "Y: " + Mojo.Format.formatNumber(parseFloat(this.ball.y), {"fractionDigits":"2"}) + "   ");
}

FirstAssistant.prototype.updateAcceleration = function (event) {
    this.ball.ax =  event.accelX;
    this.ball.ay =  event.accelY;
    this.ball.az =  event.accelZ;
}

FirstAssistant.prototype.nextLevel = function(obj) {
    this.restartGame(obj);
}

FirstAssistant.prototype.restartGame = function(obj) {
    if (obj == "quit") {
      window.close();
    }
    
    this.ball.x = 289;
    this.ball.y = 1;
    document.getElementById('myball').style.left=this.ball.x + "px"
    document.getElementById('myball').style.top=this.ball.y + "px"
    this.unpauseGame(true);
}

FirstAssistant.prototype.handleCommand = function(event) {
    this.controller=Mojo.Controller.stageController.activeScene();
    if (event.type == Mojo.Event.command) {
        switch (event.command) {
            case 'do-debug':
                if (!this.debug) {
                    this.debug = true;
                    this.dbgIntervalID = window.setInterval(this.updateDbg.bind(this),100);
                    $('mygroup').show();
                }
                else {
                    $('mygroup').hide();
                    window.clearInterval(this.dbgIntervalID);
                    this.debug = false;
                }
                break;
            case 'do-restart':
                this.restartGame(null);
                break;
            case 'do-pause':
                if (!this.paused) {
                    this.pauseGame(false);
                }
                else {
                    this.unpauseGame(false);
                }
                break;
            case 'sens-low':
                this.setFreq(4);
                break;
            case 'sens-med':
                this.setFreq(20);
                break;
            case 'sens-high':
                this.setFreq(50);
                break;
        }
    }
}
